/**
 * Created by h148225 on 2/22/2017.
 */


define(['baja!',
        'jquery',
        'Promise',
        'underscore',
        'dialogs',
        'nmodule/galileoWidgets/rc/js/GalileoUITreeView',
        'nmodule/galileoWidgets/rc/js/GalileoUITable',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/alarmlistView',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/alarmlistDisplayTable',
        'lex!sbcOfflineEngineering',
        'nmodule/sbcOfflineEngineering/rc/js/util/utils',
        'nmodule/sbcOfflineEngineering/rc/js/lib/sorttable',
        'nmodule/sbcOfflineEngineering/rc/js/lib/jquery-ui'
    ],
    function (baja,
              $,
              Promise,
              _,
              dialogs,
              treeView,
              uiTable,
              template,
              displayTableTemplate,
              Lexicons,
              utils) {

        'use strict';

        /**
         * Description of your widget.
         *
         * @class
         * @alias module:nmodule/trendMigration/rc/js/migration/MigrationSelectView
         */
        var alarmlistView = new function(){

            //define local variables and functions

            //Lexicons
            var lex = Lexicons[0];
            var delimiter = "***";

            var $element = null, $widgetContainer = null, sbcService = null;
            var soeData;

            //devices ORD path
            var alarmlistPath = "file:^offline/engineering/system.bog|bog:|slot:|";

            // globals to save device and alarm list details.
            var deviceAlarmlistData = {data:{}, _deviceMap:{}, selectedAlarms:[]};

            var treeDeviceView, alarmlistTable;

            //config
            var _configUpdateLimit = 250;
            var _dispLimit = 700;

            //prompt overwrite/progress dialog
            var overwriteDialog, _prepImportDialog;



            /**
             * For given input device (or any descandant) it will return all alarmlist part of the device or
             * selected input in device hierarchy
             * @param alarmlist
             * @param alarmlistDetail
             * @returns {*}
             */
            var _getSelectedDeviceAlarmlist = function(alarmlist, alarmlistDetail) {
                var i,j;
                for( i in alarmlistDetail) {
                    if(i === "alarmlist") {
                        for(j = 0; j < alarmlistDetail[i].length; j++) {
                            alarmlist.push(alarmlistDetail[i][j]);
                        }
                    } else if (i === "columns") {
                        continue;
                    }
                    else {
                        _getSelectedDeviceAlarmlist(alarmlist,alarmlistDetail[i]);
                    }
                }
                return alarmlist;
            };

            //checkbox handler to add selected datapoint to selected items table
            var _addDeleteSelectedAlarmlistList = function (event, checkbox) {
                var _checkbox = checkbox || this;
                var val = $(_checkbox).val();
                var tokenizer, selectColIndex, selectedRadio, selectedRadioLang = "" ;
                if($(_checkbox).is(":checked")) {
                    if(deviceAlarmlistData.selectedAlarms[val]) {
                        //already added - update info?
                    } else {
                        tokenizer = val.split(delimiter);
                        deviceAlarmlistData.selectedAlarms[val] = {
                            device:tokenizer[0],
                            symbolName:tokenizer[1],
                            selected:1
                        };

                        if(tokenizer.length > 3) {
                            deviceAlarmlistData.selectedAlarms[val]['solution'] = tokenizer[2];
                            deviceAlarmlistData.selectedAlarms[val]['project'] = tokenizer[3];
                            selectedRadio = $(_checkbox).closest("tr").find("td input[type='radio']:checked");
                            //TODO: do we always have default selected language?
                            if(selectedRadio.length > 0) {
                                deviceAlarmlistData.selectedAlarms[val]['selectedLang'] = $(selectedRadio).prop('value');
                                selectedRadioLang =  $(selectedRadio).prop('value');
                            }

                            $element.find("#soe-alarmlist-selected-table > tbody:last-child").append(
                                $("<tr class='ux-table-row' id='"+val+"'><td>"+tokenizer[1]+"</td><td title='"+tokenizer[0]+"'>"+tokenizer[0]+
                                    "</td><td>"+selectedRadioLang+"</td><td class='soe-delete-selected-icon' title='"+
                                    lex.get("sbc.offlineEngineering.alarmlist.delete")+"'></td></tr>")
                            );
                        } else {
                            $element.find("#soe-alarmlist-selected-table > tbody:last-child").append(
                                $("<tr class='ux-table-row' id='"+val+"'><td>"+tokenizer[1]+"</td><td title='"+tokenizer[0]+"'>"+tokenizer[0]+
                                    "</td><td></td><td class='soe-delete-selected-icon' title='"+
                                    lex.get("sbc.offlineEngineering.alarmlist.delete")+"'></td></tr>")
                            );
                        }
                    }
                    if($("#soe-alarmlist-display-table td input[type='checkbox']:not(:checked):first").length <= 0) {
                        $("#soe-alarmlist-display-table th input[type='checkbox']").prop("checked", true);
                    }
                } else {
                    $("#soe-alarmlist-selected-table").find("tr[id='"+utils.escapeJQID(val)+"']").remove();
                    $("#soe-alarmlist-display-table th input[type='checkbox']").prop("checked", false);
                    delete deviceAlarmlistData.selectedAlarms[val];
                }
            };

            /**
             * displaySelectedDeviceAlarmlist: create uitable of alarmlist and display them
             * @param selectedDeviceAlarmlist
             */
            var displaySelectedDeviceAlarmlist = function (deviceName, selectedDeviceAlarmlist) {

                //remove previously created table... at any point only one device points should be displayed.
                $element.find("#soe-alarmlist-display-table").remove();
                $element.find(".page-navigate").remove();

/*
                alarmlistTable = uiTable.createTable({
                    columns: deviceAlarmlistData._deviceMap.columns, rows: selectedDeviceAlarmlist,
                    options: {altStripes: false, tableId: 'soe-alarmlist-display-table', pagination:false},
                    parent: $element.find("#soe-alarmlist-display-content")
                });
*/

                $element.find("#soe-alarmlist-display-content").append($(displayTableTemplate({
                    columns: deviceAlarmlistData._deviceMap[deviceName].columns, rows: selectedDeviceAlarmlist,
                    options: {altStripes: false, tableId: 'soe-alarmlist-display-table', pagination:false}})));

                //add horizontal scroll of header
                $("#soe-alarmlist-display-table  tbody").scroll(function (e) {
                    var _self = this;
                    var left = $(_self).scrollLeft();
                    $("#soe-alarmlist-display-table .ux-table-head").scrollLeft($(_self).scrollLeft());
                });

                //wrap up - adding rows is done!
                //Display total # of points (count) for selected device
                $element.find("#soe-alarmlist-displayed-label-count").text(
                    $element.find("#soe-alarmlist-display-table tbody > tr").length);

                //restore previously selected elements if any
                if(selectedDeviceAlarmlist.length > 0) {
                    var deviceName = selectedDeviceAlarmlist[0].deviceName;
                    var t = $element.find("#soe-alarmlist-selected-table tr[id^='"+utils.escapeJQID(deviceName)+"']");
                    var inputCheckboxes = $element.find("#soe-alarmlist-display-table > tbody > tr td input[type='checkbox']");

                    $.each(t, function(index, element) {
                        var i, radioBox, eid = $(element).attr("id");
                        var selecteElem = deviceAlarmlistData.selectedAlarms[eid];
                        var chbox = inputCheckboxes.filter("[value='"+utils.escapeJQID(eid)+"']");
                        $(chbox).prop("checked", true);
                        if(selecteElem.selectedLang) {
                            radioBox = $(chbox).closest("tr").find("input[type='radio']");
                            for(i=0;i<radioBox.length;i++) {
                                if(selecteElem.selectedLang == $(radioBox[i]).val()) {
                                    $(radioBox[i]).prop("checked", true);
                                    break;
                                }
                            }
                        }

                    });

                    //if everything is checked then 'select all' header checkbox should also be checked.
                    if(inputCheckboxes.filter(":checked").length == inputCheckboxes.length)
                        $("#soe-alarmlist-display-table th input[type='checkbox']").prop("checked", true);

                }

                $element.find("#soe-alarmlist-display-table").on("click", function (event) {
                    //check for radio only
                    var tokenizer;
                    var target = event.target;
                    if($(target).prop("type") == 'radio') {
                        var parentTR = $(target).closest("tr");
                        var value = $(parentTR).find("input[type='checkbox']").val();
                        if(deviceAlarmlistData.selectedAlarms[value]) {
                            deviceAlarmlistData.selectedAlarms[value]['selectedLang'] = $(target).val();
                            $("#soe-alarmlist-selected-table").find("tr[id='"+utils.escapeJQID(value)+"']").find("td:nth-child(3)").text($(target).val());
                        }
                        event.stopPropagation();
                    }
                });


                //add the checkbox selection handler
                $("#alarmlistView .gcs_table").find("td input[type='checkbox']").on("click", function(event) {
                    var _checkbox = this;
                    window.setTimeout(function(event, _checkbox) {
                        _addDeleteSelectedAlarmlistList(event, _checkbox);
                        //update selected points count
                        $element.find("#soe-alarmlist-selected-label-count").text(
                            $element.find("#soe-alarmlist-selected-table > tbody > tr").length);
                    }, 0, event, _checkbox);
                });

                var $displayTable = $element.find("#soe-alarmlist-display-table");

                //handler to enable select all alarmlist with the alarmlist display table
                $displayTable.find("th[data-field-type='checkbox']").on("click", function(event) {
                    var _selectColumn = this;
                    var isChecked = $(_selectColumn).children("input[type='checkbox']").is(":checked");
                    var selectColIndex = $(_selectColumn).index();
                    var allAlarmlist = $("#soe-alarmlist-display-table")
                        .find("tr:not(.displayNone) td:nth-child("+ (selectColIndex+1) +") input[type='checkbox']");

                    $(allAlarmlist).prop("checked", isChecked);

                    window.setTimeout(function (event, _selectColumn, allAlarmlist) {
                        var isChecked = $(_selectColumn).children("input[type='checkbox']").is(":checked");
                        var selectColIndex = $(_selectColumn).index();
                        var i, val, $tableElement;
                        if(isChecked) {
                            var i, upperLimit, val, dataFormatIndex, tokenizer, selectedRadio, selectedRadioLang = "";
                            var tbl_TD = "</td><td class='soe-delete-selected-icon' title='"+
                                lex.get("sbc.offlineEngineering.alarmlist.delete")+"'></td></tr>" ;
                            var $tableElement = $element.find("#soe-alarmlist-selected-table > tbody:last-child");

                                for(i=0; i< allAlarmlist.length; i++) {
                                    val = $(allAlarmlist[i]).val();
                                        if(!deviceAlarmlistData.selectedAlarms[val]) {
                                            tokenizer = val.split(delimiter);
                                            deviceAlarmlistData.selectedAlarms[val] = {
                                                device:tokenizer[0],
                                                symbolName:tokenizer[1],
                                                selected:1
                                            };

                                            if(tokenizer.length > 3) {
                                                deviceAlarmlistData.selectedAlarms[val]['solution'] = tokenizer[2];
                                                deviceAlarmlistData.selectedAlarms[val]['project'] = tokenizer[3];
                                                selectedRadio = $(allAlarmlist[i]).closest("tr").find("td input[type='radio']:checked");
                                                //TODO: do we always have default selected language?
                                                if(selectedRadio.length > 0) {
                                                    deviceAlarmlistData.selectedAlarms[val]['selectedLang'] = $(selectedRadio).prop('value');
                                                    selectedRadioLang = $(selectedRadio).prop('value');
                                                }

                                                //TODO: append selected languauge?
                                                $tableElement.append(
                                                    $("<tr class='ux-table-row' id='"+val+"'><td>"+tokenizer[1]+"</td><td title='"+tokenizer[0]+"'>"+tokenizer[0]+
                                                        "<td>"+selectedRadioLang+"</td>" + tbl_TD));
                                            } else {
                                                $tableElement.append(
                                                    $("<tr class='ux-table-row' id='"+val+"'><td>"+tokenizer[1]+"</td><td title='"+tokenizer[0]+"' >"+tokenizer[0]+
                                                       "<td></td>" + tbl_TD));
                                            }
                                        }
                                }
                                //update selected points count
                                $element.find("#soe-alarmlist-selected-label-count").text(
                                    $tableElement.children().length);
                        } else {
                            $tableElement = $element.find("#soe-alarmlist-selected-table > tbody:last-child");
                            for(i=0; i< allAlarmlist.length; i++) {
                                val = $(allAlarmlist[i]).val();
                                $tableElement.find("tr[id='"+utils.escapeJQID(val)+"']").remove();
                                delete deviceAlarmlistData.selectedAlarms[val];
                            }
                            //update selected points count
                            $element.find("#soe-alarmlist-selected-label-count").text(
                                $tableElement.children().length);

                        }

                    }, 0, event, _selectColumn, allAlarmlist);
                });

                //TODO: do not sort columns that doesn't need to - performance considerations!
                 $displayTable.find("thead th[data-field-type='checkbox']").addClass("sorttable_nosort");
                $displayTable.find("thead th[data-field-type='radio']").addClass("sorttable_nosort");

                $element.find(".treeView .node-loading").removeClass("node-loading");

                sorttable.makeSortable(document.getElementById("soe-alarmlist-display-table"));
                //enable temporarily disable selection while loading...
                $element.find("#soe-alarmlist-display-table input[type='checkbox']").prop("disabled", false);


            };

            /**
             * _displayAlarmlistInfo: read (from js object) and display alarmlist for selected device in hierarchy
             * @param alarmlistDetail
             */
            var _displayAlarmlistInfo = function (deviceName, alarmlistDetail) {
                var selectedDeviceAlarmlist =  _getSelectedDeviceAlarmlist([], alarmlistDetail);
                displaySelectedDeviceAlarmlist(deviceName, selectedDeviceAlarmlist);
            };

            var _parseWebEditor8Alarmlist = function (currentTarget, deviceName, solutions) {

                var langColumns = {};
                var _deviceHrchy = {};
                _deviceHrchy[deviceName] = {name:deviceName, path:deviceName, children: []};

                var ddata = deviceAlarmlistData._deviceMap[deviceName];

                ddata.columns = [
                    {name:"", type:"checkbox", field:"select"},
                    {name:lex.get("sbc.offlineEngineering.alarmlist.alarmName"), field:"alarmName"},
                ];

                var dhrchy = _deviceHrchy[deviceName];

                var solHrchy, sol, solName;
                var projects, prjHrchy, prj, prjName;
                var spAlarmlist, al, alName, alType, alLang, alLoc, alMem, alSelLang;

                var i, j, k;
                for(i=0;i<solutions.length; i++) {
                    sol =$(solutions[i]);
                    solName = $(sol).attr("solutionName");
                    solHrchy = {name:solName, path:deviceName+"."+solName, children:[]};
                    ddata[solName] = {};
                    projects = $(sol).find("Project");

                    for(j=0;j < projects.length; j++) {
                        prj = $(projects[j]);
                        prjName = $(prj).attr("projectName");
                        prjHrchy = {name:prjName, path:deviceName +"."+solName +"."+prjName, children:[]};
                        ddata[solName][prjName] = {alarmlist : []};

                        spAlarmlist = $(prj).find("AlarmList");
                        for(k=0; k < spAlarmlist.length; k++) {
                            al = $(spAlarmlist[k]);
                            alName = $(al).attr("symbolName");
                            alLang = $(al).attr("languages");
                            alSelLang = $(al).attr("selectedLanguage");

                            var alarmListData = {
                                select:deviceName+delimiter+alName+delimiter+solName+delimiter+prjName,
                                alarmName:alName,
                                alarmNameClass:alName,
                                alarmType:$(al).attr("type"),
                                deviceName:deviceName,
                                alarmLoc: $(al).attr("location"),
                                memoryAddr: $(al).attr("memoryAddress"),
                                languages:{}
                            };
                            if(alLang && alLang != "NONE") {
                                var tokenizeLang = alLang.split(',');
                                var tokenLang, tlang, tl;
                                for(tokenLang =0; tokenLang < tokenizeLang.length; tokenLang++) {
                                    tl =tokenizeLang[tokenLang];
                                    if(!langColumns[tl])
                                        langColumns[tl] = true;
                                    if(tl == alSelLang) {
                                        alarmListData.languages[tl] = {applicable:true, name:tl, selected : true};
                                    } else {
                                        alarmListData.languages[tl] = {applicable:true, name:tl};
                                    }
                                }
                            }
                            ddata[solName][prjName].alarmlist.push(alarmListData);
                        }

                        solHrchy.children.push(prjHrchy);
                    }
                    dhrchy.children.push(solHrchy);
                }
                for(i in langColumns) {
                    ddata.columns.push({
                        name:i, type:"radio", field:i, radio:true});
                }

                //Displaying alarmType as last column.
                ddata.columns.push({name:lex.get("sbc.offlineEngineering.alarmlist.alarmType"), field:"alarmType"});


                treeDeviceView.addChildren(currentTarget, _deviceHrchy[deviceName].children);
                treeDeviceView.expandTreeNode($(currentTarget).children("div").find(".expandcollapse"));
            };


            var _parseDDCSuiteAlarmlist = function (currentTarget, deviceName, spAlarmlist) {

                deviceAlarmlistData._deviceMap[deviceName] = {};
                var ddata = deviceAlarmlistData._deviceMap[deviceName];
                var al, alName, alType, alLoc, alMem;
                var k;

                ddata.columns =  [
                    {name:"", type:"checkbox", field:"select"},
                    {name:lex.get("sbc.offlineEngineering.alarmlist.alarmName"), field:"alarmName"},
                    {name:lex.get("sbc.offlineEngineering.alarmlist.alarmType"), field:"alarmType"}
                ];
                ddata.alarmlist = [];
                for(k=0; k < spAlarmlist.length; k++) {
                    al = $(spAlarmlist[k]);
                    alName = $(al).attr("symbolName");

                    var alarmListData = {
                        select:deviceName+delimiter+alName,
                        alarmName:alName,
                        alarmNameClass:alName,
                        alarmType: $(al).attr("type"),
                        deviceName:deviceName,
                        alarmLoc: $(al).attr("location"),
                        memoryAddr: $(al).attr("memoryAddress")
                };
                    ddata.alarmlist.push(alarmListData);
                }
            };


            /**
             * _getAndDisplayAlarmlistInfo - get alarmlist for selected device by reading the corresponding device xml
             * and store & display read alarmlist
             * @param currentTarget
             * @private
             */
            var _getAndDisplayAlarmlistInfo = function (currentTarget) {

                var  filename =  $(currentTarget).attr("data-deviceFile");

                //Note: check if there is device xml
                if(filename != null && filename.length > 0 && filename.trim() != "null") {
                    baja.Ord.make(filename).get( {
                        ok:function(result) {
                            $.get(result.getReadUri(), function(data) {

                                //common device and table header info for both webeditor8 and ddcsuite
                                var device = $(data).find("Device");
                                var deviceName = $(device).attr("name");
                                //column order point name --> media type --> address --> data format --> comment
                                deviceAlarmlistData._deviceMap[deviceName] = {};

                                var solutions = $(data).find("Solution");
                                if(solutions.length > 0) {
                                    //WebEditor8 Alarmlist
                                    _parseWebEditor8Alarmlist(currentTarget, deviceName, solutions);
                                } else {
                                    var spAlarmlist = $(data).find("AlarmList");
                                    _parseDDCSuiteAlarmlist(currentTarget, deviceName, spAlarmlist);
                                }
                                _displayAlarmlistInfo(deviceName, deviceAlarmlistData._deviceMap[deviceName]);
                            });
                        }
                    });
                } else {
                    //simply remove the previous table  - if there is no datapoints to show, let the space be empty
                    $element.find("#soe-alarmlist-display-content").empty();
                    $element.find("#soe-alarmlist-displayed-label-count").text("0");
                    //No spinner for alarms
                    //$element.find(".treeView .spinner").removeClass("spinner");
                }
            };


            var _getAlarmlistInfo = function (event) {
                var currentTarget, targetElement, targetLI, targetKey, key, data, tokens, dlevel;
                currentTarget = event.currentTarget;
                $element.find('.treeView .tree-node-selected').removeClass('tree-node-selected');
                $(currentTarget).children("div").addClass('tree-node-selected node-loading');
                key = $(currentTarget).attr("data-key");
                var deviceName = (key.split("."))[0]

                if(key && deviceAlarmlistData._deviceMap[deviceName]) {
                    //take the toElement key
                    targetElement  = event.target;
                    targetLI = $(targetElement).closest("li");
                    targetKey = $(targetLI).attr('data-key');
                    tokens = targetKey.split(".");
                    dlevel = 0;
                    data = deviceAlarmlistData._deviceMap;

                    //get the right device/project/solution from hierarchy to display alarmlist
                    while(dlevel < tokens.length) {
                        data = data[tokens[dlevel]];
                        dlevel++;
                    }

                    //display alarmlist for selected device/project/solution and all its descendants
                    _displayAlarmlistInfo(deviceName, data);

                } else {
                    _getAndDisplayAlarmlistInfo(currentTarget);
                }
                event.stopPropagation();
            };

            /**
             *_getDeviceData: get scanned list of devices to import from sbcOfflineEngineering service
             * @private
             *
             */
            var _getDeviceData = function() {
                var dataOrd = alarmlistPath +"bql:select deviceName, alarmListFile, ipAddress, stationNumber, numberOfAlarmLists from sbcOfflineEngineering:SbcOfflineDevice order by deviceName";

                try {
                    baja.Ord.make(dataOrd)
                        .get({
                            ok: function (result) {
                                //returns resolved ord target
                            },
                            cursor: {
                                each: function () {
                                    var schematicsCursor = this;
                                    var slotPath = schematicsCursor.get("slotPath") ? schematicsCursor.get("slotPath") : "";
                                    var deviceName = schematicsCursor.get("deviceName") ? schematicsCursor.get("deviceName") : "";
                                    var alarmListFile = schematicsCursor.get("alarmListFile") ? schematicsCursor.get("alarmListFile") : "";
                                    var ipAddress = schematicsCursor.get("ipAddress") ? schematicsCursor.get("ipAddress") : "";
                                    var stationNumber = schematicsCursor.get("stationNumber") ? schematicsCursor.get("stationNumber") : "";
                                    var totalAlarmLists = schematicsCursor.get("numberOfAlarmLists") ? schematicsCursor.get("numberOfAlarmLists") : 0;
                                    var deviceNameTitle = lex.get("sbc.offlineEngineering.datapoints.ipAddress") +
                                        " " + ipAddress + "\n" +
                                        lex.get("sbc.offlineEngineering.datapoints.stationNumber") + " " + stationNumber;
                                    if(totalAlarmLists!=0) {
                                        deviceAlarmlistData.data[deviceName] = {
                                            "slotPath": slotPath,
                                            "deviceName": deviceName,
                                            "deviceNameTitle": deviceNameTitle,
                                            "deviceFile":alarmListFile.$ord
                                        };
                                    }
                                },
                                after: function () {
                                    try {
                                        treeDeviceView = treeView.create({ data:deviceAlarmlistData.data,
                                            parent:$element.find("#soe-alarmlist-device-content"),
                                            onclick: _getAlarmlistInfo, options:{id:"soe-alarmlist-device-content-treeView"}
                                        });

                                        //update total device count
                                        $element.find("#soe-alarmlist-devices-label-count").text(
                                            $element.find("#soe-alarmlist-device-content-treeView.treeView > ul > li").length);

                                        //display first device alarmlist by default... highlight the selection!!
                                        $element.find("#soe-alarmlist-device-content-treeView .tree-node:first > div .expandcollapse").click();

                                    } catch (e) {
                                        // logWarning(e);
                                    }
                                },
                                limit: 1000, //soeData.scannedList.alarms /* get scanned alarms count!! */
                                offset: 0
                            }
                        });
                } catch (e) {
                    console.log(e);
                }
            };

            var _resetData = function() {
                //TODO: reset selected data
                soeData = {};
                $element = null;
                $widgetContainer = null;
                sbcService = null;

                // globals to save device and alarm list details.
                deviceAlarmlistData = {data:{}, _deviceMap:{}, selectedAlarms:[]};
                treeDeviceView = null;
                overwriteDialog = null;
                _prepImportDialog = null;
            };

            var _addHandlerToResizeTables = function () {
                //handler to maximize and restore alarmlist display table for readability
                $element.find("#soe-alarmlist-display-restore-max").on('click', function() {
                    var restoreMaxIcon = this;
                    if($(restoreMaxIcon).hasClass("restored")) {
                        $element.find("#soe-alarmlist-selection-alarmlist").switchClass("displayDiv animateWidth31", "hideDiv", 10);
                        $element.find("#soe-alarmlist-display-alarmlist").switchClass("hideDiv animateWidth44", "displayDiv animateWidth75", 10);
                        $(restoreMaxIcon).removeClass("restored");
                        $(restoreMaxIcon).addClass("maximized");
                        $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/restore.svg');

                    } else if($(restoreMaxIcon).hasClass("maximized")) {
                        $element.find("#soe-alarmlist-display-alarmlist").switchClass("animateWidth75", "animateWidth44", 10);
                        $element.find("#soe-alarmlist-selection-alarmlist").switchClass("hideDiv","displayDiv animateWidth31", 10);
                        $(restoreMaxIcon).removeClass("maximized");
                        $(restoreMaxIcon).addClass("restored");
                        $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/maximize.svg');

                    }
                });

                //handler to maximize and restore alarmlist selection table for readability
                $element.find("#soe-alarmlist-selection-restore-max").on('click', function() {
                    var restoreMaxIcon = this;
                    if($(restoreMaxIcon).hasClass("restored")) {
                        $element.find("#soe-alarmlist-display-alarmlist").switchClass("displayDiv animateWidth44", "hideDiv", 10);
                        $element.find("#soe-alarmlist-selection-alarmlist").switchClass("hideDiv animateWidth31", "displayDiv animateWidth75", 10);
                        $(restoreMaxIcon).removeClass("restored");
                        $(restoreMaxIcon).addClass("maximized");
                        $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/restore.svg');


                    } else if($(restoreMaxIcon).hasClass("maximized")) {
                        $element.find("#soe-alarmlist-selection-alarmlist").switchClass("animateWidth75", "animateWidth31", 10);
                        $element.find("#soe-alarmlist-display-alarmlist").switchClass("hideDiv", "displayDiv animateWidth44", 10);
                        $(restoreMaxIcon).removeClass("maximized");
                        $(restoreMaxIcon).addClass("restored");
                        $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/maximize.svg');
                    }
                });

            };



            /**
             * _updateConfigSelection: create baja component for user selected datapoints
             * @param selectedList
             * @returns {Array}
             * @private
             */
            var _updateConfigSelection = function (selectedList) {
                var dataArray = [];
                var mainComponent, datapointComponent, tempComponent;

                if(soeData.scannedList.alarms){
                    var hasMoreAlarms = true;
                    var selectedListKeys = Object.keys(selectedList);
                    var i, lowerLimit = 0, upperLimit = (selectedListKeys.length < _configUpdateLimit) ? selectedListKeys.length : _configUpdateLimit;
                    while(hasMoreAlarms) {
                        mainComponent = new baja.Component();
                        datapointComponent = new baja.Component();
                        for (i = lowerLimit; i < upperLimit; i++) {
                            tempComponent = new baja.Component();
                            tempComponent.add({slot:'device', value:selectedList[selectedListKeys[i]].device});
                            tempComponent.add({slot:'pointName', value:selectedList[selectedListKeys[i]].symbolName});
                            // tempComponent.add({slot:'selected', value:selectedList[selectedListKeys[i]].selected});
                            tempComponent.add({slot:'overwrite', value:selectedList[selectedListKeys[i]].overwrite});

                            //Note: Doesn't solution ensures project and lang are available, doesn't it means its WE8?
                            if(selectedList[selectedListKeys[i]].solution)
                                tempComponent.add({slot:'solutionName', value:selectedList[selectedListKeys[i]].solution});
                            if(selectedList[selectedListKeys[i]].project)
                                tempComponent.add({slot:'projectName', value:selectedList[selectedListKeys[i]].project});
                            if(selectedList[selectedListKeys[i]].selectedLang)
                                tempComponent.add({slot:'selectedLanguage', value:selectedList[selectedListKeys[i]].selectedLang});
                            datapointComponent.add(tempComponent);
                        }
                        mainComponent.add( {slot:"alarmlists", value:datapointComponent});
                        var startPosition = "1",endPosition="0";
                        if(lowerLimit==0)
                             startPosition = "0"
                        if(upperLimit==selectedListKeys.length)
                        endPosition = "1"
                        mainComponent.add( {slot:"startPosition", value:startPosition});
                        mainComponent.add( {slot:"endPosition", value:endPosition});
                        dataArray.push(mainComponent);

                        if(upperLimit < selectedListKeys.length) {
                            lowerLimit = upperLimit;
                            upperLimit = (selectedListKeys.length < (upperLimit+_configUpdateLimit)) ? selectedListKeys.length : (upperLimit+_configUpdateLimit);
                        } else {
                            hasMoreAlarms = false;
                        }
                    }
                }

                return Promise.map(dataArray, function(mainComponent) {
                    return  sbcService.invoke({
                        slot:"updateAlarmListInformation",
                        value:mainComponent
                    });
                });
            };


            /**
             *
             * @param resolve
             * @param hasMoreOverwritePrompts
             * @param keys
             * @param index
             * @private
             */
            var _promptOverwriteDialog = function (resolve, hasMoreOverwritePrompts, keys, index) {

                var content = "<div class='soe-overwrite-dialog'><div>"+
                    lex.get("sbc.offlineEngineering.alarmlist.overwrite")+"</div>" +
                    "<div> <input type='checkbox' id='soe-datapoints-overwrite-check' >" +
                    "<label for='soe-datapoints-overwrite-check'>" +
                    lex.get("sbc.offlineEngineering.alarmlist.overwrite.repeat")+"</label>" +
                    "</div></div>";

                content = content.replace("$$deviceName$$", (deviceAlarmlistData.selectedAlarms[keys[index]]).device)
                    .replace("$$alarmlistName$$", (deviceAlarmlistData.selectedAlarms[keys[index]]).symbolName);

                overwriteDialog =  dialogs.make(
                    {
                        title : lex.get("sbc.offlineEngineering.dialog.confirm"),
                        content:content,
                        buttons: [
                            {
                                name:"overwrite",
                                displayName: lex.get("sbc.offlineEngineering.dialog.overwrite"),
                                handler: function(dlg) {
                                    deviceAlarmlistData.selectedAlarms[keys[index]].overwrite='1';
                                    index++;
                                    var dlgContent = dlg.content();
                                    dlg.close();
                                    var repeatYes = $(dlgContent).find("#soe-datapoints-overwrite-check").prop("checked");

                                    if(repeatYes || index >= keys.length ) {
                                        _prepImportDialog = dialogs.show({
                                            title:lex.get("sbc.offlineEngineering.dialog.info"),
                                            content: lex.get("sbc.offlineEngineering.dialog.prepareforimport")});

                                        //update user selection in case repeatYes and there are more than one datapoint
                                        // pending overwrite update.
                                        while (index < keys.length) {
                                            deviceAlarmlistData.selectedAlarms[keys[index++]].overwrite='1';
                                        }

                                        return new Promise(function (resolve, reject) {
                                            //reset datapoints selection before update
                                            var alarmsContainer = sbcService.getAlarms();
                                            alarmsContainer.setSelectedAlarmListCount(0);
                                            var selectedList = {};
                                            if(soeData.scannedList.alarms) {
                                                selectedList = deviceAlarmlistData.selectedAlarms;
                                            }
                                            resolve(selectedList);
                                        }).then(function (selectedList) {
                                                return _updateConfigSelection(selectedList);
                                            }). then(function(){
                                            if(_prepImportDialog && !_prepImportDialog.isClosed()) {
                                                _prepImportDialog.close();
                                            }
                                            resolve();
                                        });
                                    } else {
                                        _promptOverwriteDialog(resolve, hasMoreOverwritePrompts, keys, index);
                                    }
                                }
                            },
                            {
                                name:"skip",
                                displayName:lex.get("sbc.offlineEngineering.dialog.skip"),
                                handler:function (dlg) {
                                    deviceAlarmlistData.selectedAlarms[keys[index]].overwrite='0';
                                    index++;
                                    dlg.close();
                                    var dlgContent = dlg.content();
                                    var repeatNo = $(dlgContent).find("#soe-datapoints-overwrite-check").prop("checked");
                                    if(repeatNo) {
                                        while (index < keys.length) {
                                            deviceAlarmlistData.selectedAlarms[keys[index++]].overwrite='0';
                                        }
                                    }
                                    if(repeatNo || index >= keys.length ) {

                                        _prepImportDialog = dialogs.show({
                                            title:lex.get("sbc.offlineEngineering.dialog.info"),
                                            content: lex.get("sbc.offlineEngineering.dialog.prepareforimport")});

                                        //update user selection in case repeastNo and there are more than one datapoint
                                        // pending overwrite update.
                                        while (index < keys.length) {
                                            deviceAlarmlistData.selectedAlarms[keys[index++]].overwrite='0';
                                        }
                                        return new Promise(function (resolve, reject) {
                                            //reset datapoints selection before update
                                            var alarmsContainer = sbcService.getAlarms();
                                            alarmsContainer.setSelectedAlarmListCount(0);
                                            var selectedList = {};
                                            if(soeData.scannedList.alarms) {
                                                selectedList = deviceAlarmlistData.selectedAlarms;
                                            }
                                            resolve(selectedList);
                                        }).then(function (selectedList) {
                                            return _updateConfigSelection(selectedList);
                                        }). then(function(){
                                            if(_prepImportDialog && !_prepImportDialog.isClosed()) {
                                                _prepImportDialog.close();
                                            }
                                            resolve();
                                        });
                                    } else {
                                        _promptOverwriteDialog(resolve, hasMoreOverwritePrompts, keys, index);
                                    }
                                }
                            }
                        ]
                    });
                overwriteDialog.show({content:content});
            };


            return {


                /**
                 *
                 * @param element: dom element the view will be added to
                 * @param params: javascript object with widget parameters required to load the view within the widget
                 */

                loadView : function(element, params) {
                    _resetData();
                    var _self = this;
                    $element = $(element);
                    $widgetContainer = (params.widgetContainer) ?  $(params.widgetContainer) : (element).closest(".sbcOfflineEngineering-wizard");
                    soeData.selectedModules = params.selectedModules;
                    soeData.scannedList = params.scannedModules;
                    sbcService = params.component;

                    //append default html layout template for alarmlist selection screens
                    $(element).append(template({
                        defaultActionMsg: lex.get("sbc.offlineEngineering.alarmlist.defaultMsg"),
                        deviceSelectAll:lex.get("sbc.offlineEngineering.alarmlist.deviceSelectAll"),
                        alarmlistDeviceCount:lex.get("sbc.offlineEngineering.alarmlist.alarmlistDeviceCount"),
                        alarmlistSelectAll:lex.get("sbc.offlineEngineering.alarmlist.alarmlistSelectAll"),
                        alarmlistCount:lex.get("sbc.offlineEngineering.alarmlist.alarmlistCount"),
                        alarmlistSelectedPoints:lex.get("sbc.offlineEngineering.alarmlist.alarmlistSelectedPoints"),
                        alarmlistDeleteAll:lex.get("sbc.offlineEngineering.alarmlist.alarmlistDeleteAll"),
                        alarmName:lex.get("sbc.offlineEngineering.alarmlist.alarmName"),
                        expandCollapseMsg:lex.get("sbc.offlineEngineering.alarmlist.svg.altText"),
                        device: lex.get("sbc.offlineEngineering.alarmlist.device"),
                        language: lex.get("sbc.offlineEngineering.alarmlist.language")
                    }));


                    //handler to delete item from selected alarmlist list
                    //Note: Attach handler only once - as it delegates the deletion to table we don't have to attach it multiple times
                    $element.find("#soe-alarmlist-selected-table").on("click", function (event) {
                        //check for delete only
                        var tokenizer;
                        var target = event.target;

                        if($(target).hasClass("soe-delete-selected-icon")) {
                            var parentId = $(target).parent().attr("id");
                            tokenizer = parentId.split(delimiter);
                            $("#soe-alarmlist-selected-table").find("tr[id='" + utils.escapeJQID(parentId)+"']").remove();
                            var selectedInput = $("#soe-alarmlist-display-table")
                                .find("td.gcs_td_cell input[type='checkbox'][value='"+parentId+"']");
                            $(selectedInput).prop("checked", false);
                            delete deviceAlarmlistData.selectedAlarms[parentId];
                            //update selected points count
                            $element.find("#soe-alarmlist-selected-label-count").text(
                                $element.find("#soe-alarmlist-selected-table > tbody > tr").length);
                            var displayedTR = $("#soe-alarmlist-display-table tbody tr");
                            if(displayedTR.length > displayedTR.find("input[type='checkbox']:checked").length) {
                                $("#soe-alarmlist-display-table").
                                find("thead th[data-field-type='checkbox'] input").prop("checked", false);
                            }
                            event.stopPropagation();
                        }
                    });

                    //add horizontal scroll of header
                    $("#soe-alarmlist-selected-table  tbody").scroll(function (e) {
                        var _self = this;
                        var left = $(_self).scrollLeft();
                        $("#soe-alarmlist-selected-table .ux-table-head").scrollLeft($(_self).scrollLeft());
                    });


                    $element = $(element).find("#" + _self.params.id);

                    //Attach handlers  to maximize and restore alarmlist display and selection tables for readability
                    _addHandlerToResizeTables();

                    //Read scanned devices from sbcOfflineEngineering service and display alarmlist for 1st device.
                    _getDeviceData();
                },

                /**
                 * validate user input
                 *
                 * @returns {boolean}: return true/false based on validity of user input
                 */
                 validate: function() {
                 //TODO: validate user input
                    //validate user input
                    var i, id, radio, tokens, content, isSelected = true;
                    var selectedAlarmlist = $element.find("#soe-alarmlist-selected-table tbody > tr");
                    return new Promise(function (resolve) {
                        if(selectedAlarmlist.length <= 0) {
                            dialogs.showOk({
                                title : lex.get("sbc.offlineEngineering.dialog.error"),
                                content:lex.get("sbc.offlineEngineering.alarmlist.selectinput")
                            });
                            resolve(false);
                        } else {
                            for(i=0; i < selectedAlarmlist.length; i++) {
                                id = $(selectedAlarmlist[i]).attr("id");
                                //radio =  $("#soe-alarmlist-display-table").find("td input[type='radio']:first");
                                //TODO: do we always have default selected language?
                                //TODO: handle ddc suite where default languauge might not be present
                                if(!deviceAlarmlistData.selectedAlarms[id]['selectedLang']) {
                                    tokens = id.split(delimiter);
                                    content = lex.get("sbc.offlineEngineering.alarmlist.selectRadio");
                                    content = content.replace("$$deviceName$$", tokens[0])
                                        .replace("$$alarmlistName$$", tokens[1]);
                                    isSelected = false;
                                    break;
                                }
                            }
                            if(!isSelected) {
                                dialogs.showOk({
                                    title : lex.get("sbc.offlineEngineering.dialog.error"),
                                    content: content
                                });
                                resolve(false);
                            } else {
                                resolve(true);
                            }
                        }
                    });
                 },

                /**
                 * clean up and save before going to next view
                 *
                 * @returns {Promise}: return promise to be resolved before going to next view
                 */
                next : function() {
                    return new Promise(function(resolve){
                        //TODO:task to do before moving to next screen
                        var hasMoreOverwritePrompts = true;
                        var index = 0;
                        var keys = Object.keys(deviceAlarmlistData.selectedAlarms);

                        _promptOverwriteDialog(resolve, hasMoreOverwritePrompts, keys, index);

                        //resolve();

                    });
                },

                /**
                 * clean up and save before going to previous view
                 *
                 */
                back : function() {
                    return new Promise(function(resolve){
                        //TODO:task to do before moving to previous screen
                        resolve();
                    });
                },

                /**
                 * cancel operation
                 *
                 */
                 cancel: function() {
                 //TODO: cancel running operation within current view
                    //Note: for DatapointsView cancel is similar to reset selection functionality
                    //remove all selected datapoints <tr>
                    $element.find("#soe-alarmlist-selected-table > tbody > tr").remove();
                    //uncheck points for the selected device in display table
                    $element.find("#soe-alarmlist-display-table th > input[type='checkbox']").prop("checked", false);
                    $element.find("#soe-alarmlist-display-table  > tbody > tr td.gcs_td_cell input[type='checkbox']")
                        .prop("checked", false);
                    //clear selected datapoints list saved internally
                    deviceAlarmlistData.selectedAlarms = {};
                    //reset selected points count to zero
                    $element.find("#soe-alarmlist-selected-label-count").text(
                        $element.find("#soe-alarmlist-selected-table > tbody > tr").length);
                },

                /**
                 * data to be passed to widget to communicate between views
                 *
                 * @returns {json}: return key value pairs of data
                 */
                getData: function() {
                    //TODO: task to update data to be passed to widget
                    return {};
                },


                /**
                 * enable/disable status of visisble action buttons within the view
                 *
                 * @returns {json}: return action buttons status (enabled/disabled)
                 */
                /*
                 //Note: by default all buttons within the view are enabled.
                 // uncomment and update the code to conditionally enable or disable action buttons
                 enableDisableActionBttns: function () {
                 return {
                 disabledBack : false,
                 disabledNext : false,
                 disabledCancel:false
                 };
                 },
                 */
                params:{
                    vname:'alarmlistView',
                    id:'alarmlistView',
                    message:lex.get("sbc.offlineEngineering.alarmlist.defaultMsg"),
                    backBttnMsg: lex.get('sbc.offlineEngineering.back'),
                    nextBttnMsg: lex.get('sbc.offlineEngineering.next'),
                    isCancelAvailable: true,
                    cancelBttnMsg: lex.get('sbc.offlineEngineering.alarmlist.alarmlistResetSelection')
                }
            };
        };

        return alarmlistView;
    });