/**
 * Created by e855709 on 2/21/2017.
 */
define(['baja!',
    'jquery',
    'Promise',
    'underscore',
    'dialogs',
    'nmodule/galileoWidgets/rc/js/GalileoUITreeView',
    'nmodule/galileoWidgets/rc/js/GalileoUITable',
    'nmodule/sbcOfflineEngineering/rc/js/util/utils',
    'hbs!nmodule/sbcOfflineEngineering/rc/hbs/hdlogView',
    'lex!sbcOfflineEngineering',
    'nmodule/sbcOfflineEngineering/rc/js/lib/sorttable',
    'nmodule/sbcOfflineEngineering/rc/js/lib/jquery-ui'
], function(baja,
            $,
            Promise,
            _,
            dialogs,
            treeView,
            uiTable,
            utils,
            template,
            lexicons) {
    'use strict';

    var hdlogView = new function() {

        //define local variables and functions
        var $element = null, $widgetContainer = null, sbcService = null;
        var selectedModules = {}, scannedList = {} ;

        var lex = lexicons[0];

        var treeDeviceView = null;
        var hdlogsPath = "file:^offline/engineering/system.bog|bog:|slot:|";
        var _deviceMap = {};
        var hdlogsTable = null;
        var selectedHdlogsList = {};
        var overwriteDialog = null;
        var deviceHdlogsData = {data:[]};
        var _prepImportDialog = null;
        var _configUpdateLimit = 250;
        var _dispLimit = 700;
        var delimeter = "***";

        //return true if hdlogs are available on client-side for given device
        var _hasHdlogsData = function (key) {
            return (_deviceMap[key]) ? true :  false;
        };

        /**
         * return saved information for given key (device) in _deviceMap
         * @param key
         * @returns {{}}
         * @private
         */
        var _getSelectedDeviceInfo = function (key) {
            var tokens = key.split(".");
            var dlevel = 0;
            var ddata = _deviceMap;
            while(dlevel < tokens.length) {
                ddata = ddata[tokens[dlevel]];
                dlevel++;
            }
            return ddata;
        };

        /**
         * For given input device (or any descandant) it will return all hdlogs part of the device or
         * selected input in device hierarchy
         * @param hdlogs
         * @param hdlogsDetail
         * @returns {*}
         */
        var getSelectedDeviceHdlogs = function(hdlogs, hdlogsDetail) {
            var i,j;
            for( i in hdlogsDetail) {
                if(i === "hdlogs") {
                    for(j = 0; j < hdlogsDetail[i].length; j++) {
                        hdlogs.push(hdlogsDetail[i][j]);
                    }
                }
                else {
                    getSelectedDeviceHdlogs(hdlogs,hdlogsDetail[i]);
                }
            }
            return hdlogs;
        };

        //checkbox handler to add selected HDlog to selected items table
        var _addDeleteSelectedHdlogsList = function (event, checkbox) {
            var _checkbox = checkbox || this;
            var val = $(_checkbox).val();

            var tokenizer;
            if($(_checkbox).is(":checked")) {
                if(selectedHdlogsList[val]) {
                    //already added - update info?
                } else {
                    tokenizer = val.split(delimeter);
                    selectedHdlogsList[val] = {
                        device:tokenizer[0],
                        hdlogName:tokenizer[1],
                        selected:1
                    };
                    $element.find("#soe-hdlogs-selected-table > tbody:last-child").append(
                        $("<tr class='ux-table-row' id='"+val+"'><td>"+tokenizer[1]+"</td><td title='"+tokenizer[0]+"'>"+tokenizer[0]+
                            "</td><td class='soe-delete-selected-icon' title='"+
                            lex.get("sbc.offlineEngineering.hdlogs.delete")+"'></td></tr>")
                    );
                }
            } else {
                $("#soe-hdlogs-selected-table").find("tr[id='" + utils.escapeJQID(val)+"']").remove();
                delete selectedHdlogsList[val];
            }
        };

        var _displaySelectedDeviceHDLogsInBites = function(selectedDeviceHDLogs, index) {

            if(index >= selectedDeviceHDLogs.length) {
                //wrap up - adding rows is done!
                //Display total # of points (count) for selected device
                $element.find("#soe-hdlog-displayed-label-count").text(
                    $element.find("#soe-hdlogs-display-table tbody > tr").length);

                //restore previously selected elements
                var deviceName = selectedDeviceHDLogs[0].deviceName;
                var t = $element.find("#soe-hdlogs-selected-table tr[id^='"+utils.escapeJQID(deviceName)+"']");
                $.each(t, function(index, element) {
                    var eid = $(element).attr("id");
                    $element.find("#soe-hdlogs-display-table td input[value='"+utils.escapeJQID(eid)+"']").prop("checked", true);
                });

                var $hdlog = $("#soe-hdlogs-display-table tbody tr");
                if($hdlog.find("input[type='checkbox']:checked").length == $hdlog.length) {
                    $("#soe-hdlogs-display-table thead tr input[type='checkbox']").prop("checked", true);
                }

                //add the checkbox selection handler
                $("#hdlogView .gcs_table").find("td input[type='checkbox']").on("click", function(event) {
                    var _checkbox = this;
                    window.setTimeout(function(event, _checkbox) {
                        _addDeleteSelectedHdlogsList(event, _checkbox);
                        //update selected points count
                        $element.find("#soe-hdlog-selected-label-count").text(
                            $element.find("#soe-hdlogs-selected-table > tbody > tr").length);
                    }, 0, event, _checkbox);
                });

                var $displayTable = $element.find("#soe-hdlogs-display-table");

                //handler to enable select all hdlogs with the hdlogs display table
                $displayTable.find("th[data-field-type='checkbox']").on("click", function(event) {
                    var _selectColumn = this;
                    window.setTimeout(_selectAllHdlogs, 0, event, _selectColumn);
                });

                //do not sort columns that doesn't need to - performance considerations!
                $displayTable.find("thead th[data-field-type='checkbox']").addClass("sorttable_nosort");

                $element.find(".treeView .node-loading").removeClass("node-loading");

                sorttable.makeSortable(document.getElementById("soe-hdlogs-display-table"));
                //enable temporarily disable selection while loading...
                $element.find("#soe-hdlogs-display-table input[type='checkbox']").prop("disabled", false);

            } else {
                var maxIndex =  (index+_dispLimit > selectedDeviceHDLogs.length-1) ? selectedDeviceHDLogs.length : (index+_dispLimit);
                hdlogsTable.addRows(selectedDeviceHDLogs.slice(index, maxIndex));
                window.setTimeout(_displaySelectedDeviceHDLogsInBites, 0, selectedDeviceHDLogs, (maxIndex));
            }
        };

        /**
         * displaySelectedDeviceHDLogs: create uitable of hdlogs and display them
         * @param selectedDeviceHDLogs
         */
        var displaySelectedDeviceHDLogs = function (selectedDeviceHDLogs) {

            //remove previously created table... at any point only one device points should be displayed.
            $element.find("#soe-hdlogs-display-table").remove();
            $element.find(".page-navigate").remove();

            hdlogsTable = uiTable.createTable({
                columns: _deviceMap.columns, rows: selectedDeviceHDLogs.slice(0, _dispLimit),
                options: {altStripes: false, tableId: 'soe-hdlogs-display-table', pagination:false, treeTable: false},
                parent: $element.find("#soe-hdlog-display-content")
            });

            //add horizontal scroll of header
            $("#soe-hdlogs-display-table  tbody").scroll(function (e) {
                var _self = this;
                var left = $(_self).scrollLeft();
                $("#soe-hdlogs-display-table .ux-table-head").scrollLeft($(_self).scrollLeft());
            });

            //temporarily disable selection while loading...
            $element.find("#soe-hdlogs-display-table input[type='checkbox']").prop("disabled", true);

            window.setTimeout(_displaySelectedDeviceHDLogsInBites, 0, selectedDeviceHDLogs, (_dispLimit));
        };

        function _addSelectedHdlogsInBites(allHdlogs, isChecked, lowerLimit) {

            var allHdlogsLength = allHdlogs.length;
            var i, upperLimit, val, tokenizer;;
            var tbl_TD = "</td><td class='soe-delete-selected-icon' title='"+
                lex.get("sbc.offlineEngineering.hdlogs.delete")+"'></td></tr>" ;
            var $tableElement = $element.find("#soe-hdlogs-selected-table > tbody:last-child");

            if(lowerLimit < allHdlogsLength) {

                upperLimit = (lowerLimit+_dispLimit > allHdlogsLength) ? allHdlogsLength : lowerLimit+_dispLimit;
                for(i=lowerLimit; i< upperLimit; i++) {
                    val = $(allHdlogs[i]).val();

                    if(isChecked) {
                        if(!selectedHdlogsList[val]) {
                            tokenizer = val.split(delimeter);
                            selectedHdlogsList[val] = {
                                device:tokenizer[0],
                                hdlogName:tokenizer[1],
                                selected:1
                            };
                            $tableElement.append(
                                $("<tr class='ux-table-row' id='"+val+"'><td>"+tokenizer[1]+"</td><td title='"+tokenizer[0]+"'>"+tokenizer[0]+
                                    tbl_TD));
                        }
                    } else {
                        $tableElement.find("tr[id='"+utils.escapeJQID(val)+"']").remove();
                        delete selectedHdlogsList[val];
                    }
                }
                window.setTimeout(_addSelectedHdlogsInBites, 0, allHdlogs, isChecked, upperLimit);

            } else {
                //update selected points count
                $element.find("#soe-hdlog-selected-label-count").text(
                    $tableElement.children().length);
            }
        }

        //TODO: add selected elements in chunk
        function _selectAllHdlogs(event, _selectColumn) {
            var isChecked = $(_selectColumn).children("input[type='checkbox']").is(":checked");
            var selectColIndex = $(_selectColumn).index();
            var allHdlogs = $("#soe-hdlogs-display-table")
                .find("tr:not(.displayNone) td:nth-child("+ (selectColIndex+1) +") input[type='checkbox']");
            var i, val, $tableElement;
            if(isChecked) {
                _addSelectedHdlogsInBites(allHdlogs, isChecked, 0);
            } else {
                $tableElement = $element.find("#soe-hdlogs-selected-table > tbody:last-child");
                for(i=0; i< allHdlogs.length; i++) {
                    val = $(allHdlogs[i]).val();
                    $tableElement.find("tr[id='"+utils.escapeJQID(val)+"']").remove();
                    delete selectedHdlogsList[val];
                }
                //update selected points count
                $element.find("#soe-hdlog-selected-label-count").text(
                    $tableElement.children().length);

            }
        }

        /**
         * _displayHdlogsInfo: read (from js object) and display hdlogs for selected device in hierarchy
         * @param hdlogsDetail
         */
        var _displayHdlogsInfo = function (hdlogsDetail) {

            //get all hdlogs in the hierarchy of selected device
            var selectedDevicehdlogs =  getSelectedDeviceHdlogs([], hdlogsDetail);
            displaySelectedDeviceHDLogs(selectedDevicehdlogs);

        };


        /**
         * _getAndDisplayHdlogsInfo - get hdlogs for selected device by reading the corresponding device xml
         * and store & display read hdlogs
         * @param currentTarget
         * @private
         */
        var _getAndDisplayHdlogsInfo = function (currentTarget) {

            var  filename =  $(currentTarget).attr("data-deviceFile");

            //Note: check if there is device xml
            if(filename != null && filename.length > 0 && filename.trim() != "null") {
                baja.Ord.make(filename).get( {
                    ok:function(result) {
                        $.get(result.getReadUri(), function(data) {
                            var device = $(data).find("Device");
                            var deviceName = $(device).attr("name");
                            //column order point name --> media type --> address --> data format --> comment
                            _deviceMap.columns = [
                                {name:"", type:"checkbox", field:"select"},
                                {name:lex.get("sbc.offlineEngineering.hdlogs.hdlogName"), field:"hdlogName"},
                                {name:"", field:"pointsCount"}
                            ];
                            _deviceMap[deviceName] = {};

                            $.each($(data).find("HDLog"), function(p, e) {
                                var hdlogName = $(e).attr("logname");
                                var ddata = _deviceMap[deviceName];
                                var count = $(e).attr("records");//$(e).find("Point").length > 0 ? $(e).find("Point").length : 0;
                                var points = [];

                                if(count > 0) {
                                    $.each($(e).find("Point"), function(p, e) {
                                        var pointDetails = {};
                                        pointDetails.select = "";
                                        pointDetails.hdlogName = $(e).attr("pointName");
                                        points.push(pointDetails);
                                    });
                                }
                                if(!ddata.hdlogs) {
                                    ddata.hdlogs = [];
                                }
                                ddata.hdlogs.push({
                                    select: deviceName+delimeter+hdlogName,
                                    deviceName : deviceName,
                                    hdlogName: hdlogName,
                                    hdlogNameTitle: hdlogName,
                                    hdlogNameClass: hdlogName,
                                    pointsCount: "Points: "+count
                                });
                            });

                            _displayHdlogsInfo(_deviceMap[deviceName]);
                        });
                    }
                });
            } else {
                //simply remove the previous table  - if there is no datapoints to show, let the space be empty
                $element.find("#soe-hdlog-display-content").empty();
                $element.find("#soe-hdlog-displayed-label-count").text("0");
                //No spinner for alarms
                //$element.find(".treeView .spinner").removeClass("spinner");
            }
        };

        /**
         * _getHdlogsInfo: onclick handler of device (or any descendant in device hierarchy)
         * @param event
         * @private
         */
        var _getHdlogsInfo = function (event) {
            var currentTarget = event.currentTarget;
            $('.treeView .tree-node-selected').removeClass('tree-node-selected');
            $(currentTarget).children("div").addClass('tree-node-selected node-loading');
            var key = $(currentTarget).attr("data-key");
            if(_hasHdlogsData((key.split("."))[0])) {
                //take the toElement key
                var targetElement  = event.target;
                var targetLI = $(targetElement).closest("li");
                var targetKey = $(targetLI).attr('data-key');
                var data = _getSelectedDeviceInfo(targetKey);
                _displayHdlogsInfo(data);
            } else {
                _getAndDisplayHdlogsInfo(currentTarget);
            }
            event.stopPropagation();
        };

        /**
         *
         * @private
         * get scanned list of devices to import from sbcOfflineEngineering service
         */
        var _getDeviceData = function() {
            var dataOrd = hdlogsPath +"bql:select deviceName, hDLogListFile, ipAddress, stationNumber,numberOfHDLogs from sbcOfflineEngineering:SbcOfflineDevice order by deviceName";

            try {
                baja.Ord.make(dataOrd)
                    .get({
                        ok: function (result) {
                        },
                        cursor: {
                            each: function () {
                                var schematicsCursor = this;
                                var slotPath = schematicsCursor.get("slotPath") ? schematicsCursor.get("slotPath") : "";
                                var deviceName = schematicsCursor.get("deviceName") ? schematicsCursor.get("deviceName") : "";
                                var hDLogListFile = schematicsCursor.get("hDLogListFile") ? schematicsCursor.get("hDLogListFile") : "";
                                var ipAddress = schematicsCursor.get("ipAddress") ? schematicsCursor.get("ipAddress") : "";
                                var stationNumber = schematicsCursor.get("stationNumber") ? schematicsCursor.get("stationNumber") : "";
                                var totalHDLogs = schematicsCursor.get("numberOfHDLogs") ? schematicsCursor.get("numberOfHDLogs") : 0;
                                var deviceNameTitle = lex.get("sbc.offlineEngineering.datapoints.ipAddress") +
                                    " " + ipAddress + "\n" +
                                    lex.get("sbc.offlineEngineering.datapoints.stationNumber") + " " + stationNumber;
                                if(totalHDLogs!=0) {
                                    deviceHdlogsData.data.push({
                                        "slotPath": slotPath,
                                        "deviceName": deviceName,
                                        "deviceNameTitle": deviceNameTitle,
                                        "deviceFile":hDLogListFile.$ord
                                    });
                                }
                            },
                            after: function () {
                                try {
                                    treeDeviceView = treeView.create({ data:deviceHdlogsData.data,
                                        parent:$element.find("#soe-hdlogs-device-content"),
                                        onclick: _getHdlogsInfo, options:{id:"soe-hdlogs-device-content-treeView"}
                                    });

                                    //update total device count
                                    $element.find("#soe-hdlog-devices-label-count").text(
                                        $element.find("#soe-hdlogs-device-content-treeView.treeView > ul > li").length);

                                    //display first device hdlogs by default... highlight the selection!!
                                    $element.find("#soe-hdlogs-device-content-treeView .tree-node:first").click();
                                } catch (e) {
                                    //    logWarning(e);
                                }
                            },
                            limit: 1000, //scannedList.hdlogs /* get scanned hdlogs count!! */
                            offset: 0
                        }
                    });

            } catch (e) {
                console.log(e);
            }

        };

        var _resetData = function () {
            scannedList = {};
            selectedModules = {};
            deviceHdlogsData = {data:[]};
            _deviceMap = {};
            overwriteDialog = null;
            selectedHdlogsList = {};
        };

        /**
         * _resetAndUpdateConfigSelection: prompt "preparing for import..." dialog box and update selected hdlogs to
         * sbcOfflineEngineering service
         * @returns {*}
         * @private
         */
        var  _resetAndUpdateConfigSelection = function() {

            _prepImportDialog = dialogs.show({
                title:lex.get("sbc.offlineEngineering.dialog.info"),
                content: lex.get("sbc.offlineEngineering.hdlogs.dialog.prepareforimport")});

            return new Promise(function (resolve, reject) {
                //reset hdlogs selection before update
                var hdlogsContainer  = sbcService.getHdLogs();
                hdlogsContainer.setSelectedHDLogsCount(0);
                var selectedList = {};
                if(scannedList.hdlogs) {
                    selectedList = selectedHdlogsList;
                }
                resolve(selectedList);
            });
        };

        /**
         * _updateConfigSelection: create baja component for user selected hdlogs
         * @param selectedList
         * @returns {Array}
         * @private
         */
        var _updateConfigSelection = function (selectedList) {
            var dataArray = [];
            var mainComponent, hdlogComponent, tempComponent;

            if(scannedList.hdlogs){
                var hasMoreHdlogs = true;
                var selectedListKeys = Object.keys(selectedList);
                var i, lowerLimit = 0, upperLimit = (selectedListKeys.length < _configUpdateLimit) ? selectedListKeys.length : _configUpdateLimit;
                while(hasMoreHdlogs) {
                    mainComponent = new baja.Component();
                    hdlogComponent = new baja.Component();
                    for (i = lowerLimit; i < upperLimit; i++) {
                        tempComponent = new baja.Component();
                        tempComponent.add({slot:'device', value:selectedList[selectedListKeys[i]].device});
                        tempComponent.add({slot:'hdlogName', value:selectedList[selectedListKeys[i]].hdlogName});
                        tempComponent.add({slot:'selected', value:selectedList[selectedListKeys[i]].selected});
                        tempComponent.add({slot:'overwrite', value:selectedList[selectedListKeys[i]].overwrite});
                        hdlogComponent.add(tempComponent);
                    }
                    mainComponent.add( {slot:"hdlogs", value:hdlogComponent});
                    var startPosition = "1",endPosition="0";
                    if(lowerLimit==0)
                        startPosition = "0"
                     if(upperLimit==selectedListKeys.length)
                        endPosition = "1"
                    mainComponent.add( {slot:"startPosition", value:startPosition});
                    mainComponent.add( {slot:"endPosition", value:endPosition});
                    dataArray.push(mainComponent);
                    //        _invokeConfigUpdateAction(mainComponent, "siteUpdate");

                    if(upperLimit < selectedListKeys.length) {
                        lowerLimit = upperLimit;
                        upperLimit = (selectedListKeys.length < (upperLimit+_configUpdateLimit)) ? selectedListKeys.length : (upperLimit+_configUpdateLimit);
                    } else {
                        hasMoreHdlogs = false;
                    }
                }
            }

            return Promise.map(dataArray, function(mainComponent) {
                var hdlogsContainer = sbcService.getHdLogs();
                return  sbcService.invoke({
                    slot:"updateHDLogInformation",
                    value:mainComponent
                });
            });
        };

        /**
         * _updateConfigData: reset previously selected hdlogs config if any and update currently selected hdlogs
         * @param updateStatus
         * @returns {*}
         * @private
         */
        var _updateConfigData = function (updateStatus) {
            var configData = _resetAndUpdateConfigSelection();
            return configData.then(function (selectedList) {
                return _updateConfigSelection(selectedList);
            });
        };

        /**
         *
         * @param resolve
         * @param hasMoreOverwritePrompts
         * @param keys
         * @param index
         * @private
         */
        var _promptOverwriteDialog = function (resolve, hasMoreOverwritePrompts, keys, index) {

            var content = "<div class='soe-overwrite-dialog'><div>"+
                lex.get("sbc.offlineEngineering.hdlogs.overwrite")+"</div>" +
                "<div> <input type='checkbox' id='soe-hdlogs-overwrite-check' >" +
                "<label for='soe-hdlogs-overwrite-check'>" +
                lex.get("sbc.offlineEngineering.hdlogs.overwrite.repeat")+"</label>" +
                "</div></div>";

            content = content.replace("$$deviceName$$", (selectedHdlogsList[keys[index]]).device)
                .replace("$$hdlogName$$", (selectedHdlogsList[keys[index]]).hdlogName);

            overwriteDialog =  dialogs.make(
                {
                    title : lex.get("sbc.offlineEngineering.dialog.confirm"),
                    content:content,
                    buttons: [
                        {
                            name:"overwrite",
                            displayName: lex.get("sbc.offlineEngineering.dialog.overwrite"),
                            handler: function(dlg) {
                                selectedHdlogsList[keys[index]].overwrite='1';
                                index++;
                                var dlgContent = dlg.content();
                                dlg.close();
                                var repeatYes = $(dlgContent).find("#soe-hdlogs-overwrite-check").prop("checked");
                                if(repeatYes) {
                                    while (index < keys.length) {
                                        selectedHdlogsList[keys[index++]].overwrite='1';
                                    }
                                }
                                if(index >= keys.length ) {
                                    var onUpdatePromise = _updateConfigData(true);
                                    return onUpdatePromise.then(function(){
                                        if(_prepImportDialog && !_prepImportDialog.isClosed()) {
                                            _prepImportDialog.close();
                                        }
                                        resolve();
                                    });
                                } else {
                                    _promptOverwriteDialog(resolve, hasMoreOverwritePrompts, keys, index);
                                }
                            }
                        },
                        {
                            name:"skip",
                            displayName:lex.get("sbc.offlineEngineering.dialog.skip"),
                            handler:function (dlg) {
                                selectedHdlogsList[keys[index]].overwrite='0';
                                index++;
                                dlg.close();
                                var dlgContent = dlg.content();
                                var repeatNo = $(dlgContent).find("#soe-hdlogs-overwrite-check").prop("checked");
                                if(repeatNo) {
                                    while (index < keys.length) {
                                        selectedHdlogsList[keys[index++]].overwrite='0';
                                    }
                                }
                                if(index >= keys.length ) {
                                    var onUpdatePromise = _updateConfigData(true);
                                    return onUpdatePromise.then(function(){
                                        if(_prepImportDialog && !_prepImportDialog.isClosed()) {
                                            _prepImportDialog.close();
                                        }
                                        resolve();
                                    });
                                } else {
                                    _promptOverwriteDialog(resolve, hasMoreOverwritePrompts, keys, index);
                                }
                            }
                        }
                    ]
                });
            overwriteDialog.show({content:content});
        };

        /**
         * _updateDisplayOfSelectedDeviceHdlogs - Append input hdlogs
         * @param selectedDeviceHdlogs
         * @private
         */
        var _updateDisplayOfSelectedDeviceHdlogs = function (selectedDeviceHdlogs) {

            if(hdlogsTable == null) {
                hdlogsTable = uiTable.createTable({
                    columns: _deviceMap.columns, rows: selectedDeviceHdlogs,
                    options: {altStripes: false, tableId: 'soe-hdlogs-display-table'},
                    parent: $element.find("#soe-hdlog-display-content")
                });

                //add horizontal scroll of header
                $("#soe-hdlogs-display-table  tbody").scroll(function (e) {
                    var _self = this;
                    var left = $(_self).scrollLeft();
                    $("#soe-hdlogs-display-table .ux-table-head").scrollLeft($(_self).scrollLeft());
                });

            } else {
                //append information (rows) to the table
                hdlogsTable.addRows(selectedDeviceHdlogs);
            }

            //restore previously selected elements
            var deviceName = selectedDeviceHdlogs[0].deviceName;
            var t = $element.find("#soe-hdlogs-selected-table tr[id^='"+utils.escapeJQID(deviceName)+"']");
            $.each(t, function(index, element) {
                var eid = $(element).attr("id");
                $element.find("#soe-hdlogs-display-table td input[value='"+utils.escapeJQID(eid)+"']").prop("checked", true);
            });

            //handler to enable select all hdlogs with the hdlogs display table
            $element.find("#soe-hdlogs-display-table th[data-field-type='checkbox']").on("click", function(event) {
                var _selectColumn = this;
                window.setTimeout(_selectAllHdlogs, 0, event, _selectColumn);
            });
        };


        /**
         * _getAndDisplayHdlogsInfo - get hdlogs for selected device by reading the corresponding device xml
         * and store & display read hdlogs
         * @param currentTarget
         * @private
         */
        var _getAndUpdateDisplayHdlogsInfo = function (currentTarget) {

            var filename = $(currentTarget).attr("data-deviceFile");
        if(filename != null && filename.length > 0 && filename.trim() != "null") {
            baja.Ord.make(filename).get({
                ok: function (result) {
                    $.get(result.getReadUri(), function (data) {
                        var device = $(data).find("Device");
                        var deviceName = $(device).attr("name");
                        _deviceMap.columns = [
                            {name: "", type: "checkbox", field: "select"},
                            {name: lex.get("sbc.offlineEngineering.hdlogs.hdlogName"), field: "hdlogName"},
                            {name: "", field: "pointsCount"}
                        ];
                        _deviceMap[deviceName] = {};

                        $.each($(data).find("HDLog"), function (p, e) {
                            var hdlogName = $(e).attr("logname");
                            var count = $(e).attr("records");
                            var ddata = _deviceMap[deviceName];
                            if (!ddata.hdlogs) {
                                ddata.hdlogs = [];
                            }
                            ddata.hdlogs.push({
                                select: deviceName + delimeter + hdlogName,
                                deviceName: deviceName,
                                hdlogName: hdlogName,
                                hdlogNameTitle: hdlogName,
                                hdlogNameClass: hdlogName,
                                pointsCount: "Points: " + count
                            });
                        });

                        _updateDisplayOfSelectedDeviceHdlogs(getSelectedDeviceHdlogs([], _deviceMap[deviceName]));
                    });
                }
            });
        } else {
            $element.find("#soe-hdlog-display-content").empty();
            $element.find("#soe-hdlog-displayed-label-count").text(0);
            $element.find(".treeView .spinner").removeClass("spinner");
        }
        };


        /**
         * Handler to get hdlogs for all devices
         * @param event
         * @private
         */
        var _getAllHdlogs = function (event) {

            var devices = $element.find("#soe-hdlogs-device-content-treeView .treeView ul:first > li.tree-node");

            //clear the table and add them all
            hdlogsTable = null;
            $element.find("#soe-hdlogs-display-table").remove();
            $element.find(".page-navigate").remove();

            $.each(devices, function (index, element) {
                var key = $(element).attr("data-key");
                if(_hasHdlogsData(key)) {
                    var data = _getSelectedDeviceInfo(key);
                    //get all hdlogs in the hierarchy of selected device
                    var selectedDeviceHdlogs =  getSelectedDeviceHdlogs([], data);
                    _updateDisplayOfSelectedDeviceHdlogs(selectedDeviceHdlogs);
                } else {
                    _getAndUpdateDisplayHdlogsInfo($(element));
                }
            });
        };

        return {


            /**
             *
             * @param element: dom element the view will be added to
             * @param params: javascript object with widget parameters required to load the view within the widget
             */

            loadView : function(element, params) {
                _resetData();
                $element = $(element);
                $widgetContainer = (params.widgetContainer) ?  $(params.widgetContainer) : (element).closest(".sbcOfflineEngineering-wizard");
                selectedModules = params.selectedModules;
                scannedList = params.scannedModules;
                sbcService = params.component;

                //append default html layout template for hdlogs selection screens
                $(element).append(template({
                    defaultActionMsg: lex.get("sbc.offlineEngineering.hdlogs.defaultMsg"),
                    hdlogsDeviceCount:lex.get("sbc.offlineEngineering.hdlogs.hdlogsDeviceCount"),
                    device: lex.get("sbc.offlineEngineering.hdlogs.device"),
                    expandCollapseMsg:lex.get("sbc.offlineEngineering.datapoints.svg.altText"),
                    hdlogsSelected:lex.get("sbc.offlineEngineering.hdlogs.hdlogsSelected"),
                    hdlogsCount:lex.get("sbc.offlineEngineering.hdlogs.hdlogsCount"),
                    hdlogName:lex.get("sbc.offlineEngineering.hdlogs.hdlogName")
                }));

                //handler to delete item from selected hdlogs list
                //Note: Attach handler only once - as it delegates the deletion to table we don't have to attach it multiple times
                $element.find("#soe-hdlogs-selected-table").on("click", function (event) {
                    //check for delete only
                    var tokenizer;
                    var target = event.target;

                    if($(target).hasClass("soe-delete-selected-icon")) {
                        var parentId = $(target).parent().attr("id");
                        tokenizer = parentId.split(delimeter);
                        $("#soe-hdlogs-selected-table").find("tr[id='" + utils.escapeJQID(parentId)+"']").remove();
                        var selectedInput = $("#soe-hdlogs-display-table")
                            .find("td.gcs_td_cell input[type='checkbox'][value='"+parentId+"']");
                        $(selectedInput).prop("checked", false);
                        delete selectedHdlogsList[parentId];

                        //uncheck the display table header
                        var displayedTR = $("#soe-hdlogs-display-table tbody tr");
                        if(displayedTR.length > displayedTR.find("input[type='checkbox']:checked").length) {
                            $element.find("#soe-hdlogs-display-table th input[type='checkbox']").prop("checked", false);
                        }

                        //update selected hdlogs count
                        $element.find("#soe-hdlog-selected-label-count").text(
                            $element.find("#soe-hdlogs-selected-table > tbody > tr").length);
                        event.stopPropagation();
                    }

                });

                //add horizontal scroll of header
                $("#soe-hdlogs-selected-table  tbody").scroll(function (e) {
                    var _self = this;
                    var left = $(_self).scrollLeft();
                    $("#soe-hdlogs-selected-table .ux-table-head").scrollLeft($(_self).scrollLeft());
                });


                //handler to maximize and restore hdlogs display table for readability
                $element.find("#soe-hdlog-display-restore-max").on('click', function() {
                    var restoreMaxIcon = this;
                    if($(restoreMaxIcon).hasClass("restored")) {
                        $element.find("#soe-hdlog-selection-hdlog").switchClass("displayDiv animateWidth31", "hideDiv");
                        $element.find("#soe-hdlogs-display-hdlogs").switchClass("hideDiv animateWidth44", "displayDiv animateWidth75");
                        $(restoreMaxIcon).removeClass("restored");
                        $(restoreMaxIcon).addClass("maximized");
                        $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/restore.svg');
                    } else if($(restoreMaxIcon).hasClass("maximized")) {
                        $element.find("#soe-hdlogs-display-hdlogs").switchClass("animateWidth75", "animateWidth44", 1000);
                        $element.find("#soe-hdlog-selection-hdlog").switchClass("hideDiv","displayDiv animateWidth31");
                        $(restoreMaxIcon).removeClass("maximized");
                        $(restoreMaxIcon).addClass("restored");
                        $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/maximize.svg');
                    }
                });

                //handler to maximize and restore hdlogs selection table for readability
                $element.find("#soe-hdlog-selection-restore-max").on('click', function() {
                    var restoreMaxIcon = this;
                    if($(restoreMaxIcon).hasClass("restored")) {
                        $element.find("#soe-hdlogs-display-hdlogs").switchClass("displayDiv animateWidth44", "hideDiv", 10);
                        $element.find("#soe-hdlog-selection-hdlog").switchClass("hideDiv animateWidth31", "displayDiv animateWidth75", 10);
                        $(restoreMaxIcon).removeClass("restored");
                        $(restoreMaxIcon).addClass("maximized");
                        $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/restore.svg');

                    } else if($(restoreMaxIcon).hasClass("maximized")) {
                        $element.find("#soe-hdlog-selection-hdlog").switchClass("animateWidth75", "animateWidth31", 10);
                        $element.find("#soe-hdlogs-display-hdlogs").switchClass("hideDiv", "displayDiv animateWidth44", 10);
                        $(restoreMaxIcon).removeClass("maximized");
                        $(restoreMaxIcon).addClass("restored");
                        $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/maximize.svg');

                    }
                });


                _getDeviceData();

            },

            /**
             * validate user input
             *
             * @returns {boolean}: return true/false based on validity of user input
             */
            validate: function() {
                //validate user input
                return new Promise(function (resolve) {
                    if($element.find("#soe-hdlogs-selected-table tbody > tr").length <= 0) {
                        dialogs.showOk({
                            title : lex.get("sbc.offlineEngineering.dialog.error"),
                            content:lex.get("sbc.offlineEngineering.hdlogs.selectinput")
                        });
                        resolve(false);
                    } else {
                        resolve(true);
                    }
                });
            },

            /**
             * clean up and save before going to next view
             *
             * @returns {Promise}: return promise to be resolved before going to next view
             */
            next : function(event, params) {
                return new Promise(function(resolve){
                    //TODO:task to do before moving to next screen

                    var hasMoreOverwritePrompts = true;
                    var index = 0;
                    var keys = Object.keys(selectedHdlogsList);

                    _promptOverwriteDialog(resolve, hasMoreOverwritePrompts, keys, index);

                    //resolve();

                });
            },

            /**
             * clean up and save before going to previous view
             *
             */
            back : function() {
                return new Promise(function(resolve){
                    //TODO:task to do before moving to previous screen
                    resolve();
                });
            },

            /**
             * cancel operation
             *
             */
            cancel: function() {
                //Note: for HdlogsView cancel is similar to reset selection functionality
                //remove all selected hdlogs <tr>
                $element.find("#soe-hdlogs-selected-table > tbody > tr").remove();
                //uncheck points for the selected device in display table
                $element.find("#soe-hdlogs-display-table th > input[type='checkbox']").prop("checked", false);
                $element.find("#soe-hdlogs-display-table  > tbody > tr td.gcs_td_cell input[type='checkbox']")
                    .prop("checked", false);
                //clear selected hdlogs list saved internally
                selectedHdlogsList = {};
                //reset selected points count to zero
                $element.find("#soe-hdlog-selected-label-count").text(
                    $element.find("#soe-hdlogs-selected-table > tbody > tr").length);

            },

            /**
             * data to be passed to widget to communicate between views
             *
             * @returns {json}: return key value pairs of data
             */
            getData: function() {
                //Note: task to update data to be passed to widget
                return {};
            },


            /**
             * enable/disable status of visisble action buttons within the view
             *
             * @returns {json}: return action buttons status (enabled/disabled)
             */
            /*
             //Note: by default all buttons within the view are enabled.
             // uncomment and update the code to conditionally enable or disable action buttons
             enableDisableActionBttns: function () {
             //Note: one can update action buttons status
             return {
             disabledBack : false,
             disabledNext : false,
             disabledCancel:false
             };
             },
             */
            params:{
                vname:'hdlogView',
                id:'hdlogView',
                //icon:"/path/to/someViewIcon.svg",
                message: lex.get('sbc.offlineEngineering.hdlogs.defaultMsg'),
                backBttnMsg: lex.get('sbc.offlineEngineering.back'),
                nextBttnMsg: lex.get('sbc.offlineEngineering.next'),
                isCancelAvailable:true,
                cancelBttnMsg: lex.get('sbc.offlineEngineering.hdlogs.resetSelection')
            }
        };
    }();
    return hdlogView;
});
