/**
 * Created by h145414 on 12/22/2016.
 */

define(['baja!',
        'jquery',
        'Promise',
        'underscore',
        'dialogs',
        'nmodule/sbcOfflineEngineering/rc/js/util/Progress',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/importView',
        'lex!sbcOfflineEngineering'
    ],
    function (baja,
              $,
              Promise,
              _,
              Dialogs,
              progress,
              template,
              Lexicons) {

        'use strict';

        /**
         * Description of your widget.
         *
         * @class
         * @alias module:nmodule/sbcOfflineEngineering/rc/js/OfflineEngineeringimportView
         */
        var importView = new function () {

            var lex = Lexicons[0];
            var $element = null;
            var selectedModules = {};
            var importStatus = {};
            var importList = {datapoints: 0, alarms: 0, hdlogs: 0};
            var sbcService = null;
            var $widgetContainer = null;
            var isImportProcessCancelled=false;
            var cancelDialog = null;

            var _updateBttnsOnImport = function () {
                $widgetContainer.find("#sbcOfflineEngineering-cancelBttn").prop("disabled", false);
                $widgetContainer.find("#sbcOfflineEngineering-backBttn").prop("disabled", true);
            };

            var _updateBttnsOnCancel = function () {
                $widgetContainer.find("#sbcOfflineEngineering-cancelBttn").prop("disabled", true);
                $widgetContainer.find("#sbcOfflineEngineering-backBttn").prop("disabled", false);
                $widgetContainer.find("#sbcOfflineEngineering-nextBttn").css("display","inline-block");
                $widgetContainer.find("#sbcOfflineEngineering-nextBttn").prop("disabled", false);
            };

            var _startImporting = function (type) {
                _updateBttnsOnImport();
                switch (type) {
                    case 'datapoints':
                        progress.prototype.startDatapointsTimer();
                        sbcService.invoke({
                            slot: "importDataPoints"
                        });
                        break;
                    case 'alarms':
                        progress.prototype.startAlarmsTimer();
                        sbcService.invoke({
                            slot: "importAlarms"
                        });
                        break;
                    case 'hdlogs':
                        progress.prototype.startHDLogsTimer();
                        sbcService.invoke({
                            slot: "importHDLogs"
                        });
                        break;
                    default:
                        console.log("SBC service doesn't support import of " + type);
                        break;
                }
            };

            var _stopImporting = function (type) {
                switch (type) {
                    case 'datapoints':
                        var datapointsContainer = sbcService.getDataPoints();
                        importList.datapoints = datapointsContainer.getCompletedDataPointsCount();
                        progress.prototype.stopDatapointsTimer(sbcService.getIsImportCancelled() || isImportProcessCancelled);
                        break;
                    case 'alarms':
                        var alarmsContainer = sbcService.getAlarms();
                        importList.alarms = alarmsContainer.getCompletedAlarmListCount();
                        progress.prototype.stopAlarmsTimer(sbcService.getIsImportCancelled() || isImportProcessCancelled);
                        break;
                    case 'hdlogs':
                        var hdlogsContainer = sbcService.getHdLogs();
                        importList.hdlogs = hdlogsContainer.getCompletedHDLogsCount();
                        progress.prototype.stopHDLogsTimer(sbcService.getIsImportCancelled() || isImportProcessCancelled);
                        break;

                    default:
                        console.log("SBC service doesn't support import of " + type);
                }

                if (!importStatus.datapoints && !importStatus.alarms && !importStatus.hdlogs && !isImportProcessCancelled){
                    if(cancelDialog != null) {
                        cancelDialog.close();
                        cancelDialog = null;
                    }
                    $widgetContainer.find("#sbcOfflineEngineering-nextBttn").click();
                }
            };


            var _startStopImporting = function (stopImport) {

                if (stopImport) {
                    if (importStatus.datapoints) {
                        importStatus.datapoints = 0;
                        _stopImporting('datapoints');
                    } else if (importStatus.alarms) {
                        importStatus.alarms = 0;
                        _stopImporting('alarms');
                    } else if (importStatus.hdlogs) {
                        importStatus.hdlogs = 0;
                        _stopImporting('hdlogs');
                    }
                }
                if (!isImportProcessCancelled) {
                    if (importStatus.datapoints) {
                        _startImporting('datapoints');
                    }
                    else if (importStatus.alarms) {
                        _startImporting('alarms');
                    }
                    else if (importStatus.hdlogs) {
                        _startImporting('hdlogs');
                    }
                } else {
                    $widgetContainer.find("#importView #sbcOfflineEngineering-status-message").html(lex.get("sbc.offlineEngineering.cancelled.status"));
                }
            };

            var alarmListStatus = function(selected,completed){
                    progress.prototype.setAlarmsCount(completed + " of " + selected);
                    var percentage = parseInt(parseInt(completed) / parseInt(selected) * 100);
                    if (isNaN(percentage)) {
                        percentage = 0;
                    }
                    $widgetContainer.find("#pcAlarms").prop("innerHTML", (percentage + "%"));
             }

             var dataPointsStatus = function(selected,completed){
                     progress.prototype.setDatapointsCount(completed + " of " + selected);
                     var percentage = parseInt(parseInt(completed) / parseInt(selected) * 100);
                     if (isNaN(percentage)) {
                         percentage = 0;
                     }
                     $widgetContainer.find("#pcDataPoints").prop("innerHTML", (percentage + "%"));
              }

              var hdLogsStatus = function(selected,completed){
                      progress.prototype.setHDLogsCount(completed + " of " + selected);
                      var percentage = parseInt(parseInt(completed) / parseInt(selected) * 100);
                      if (isNaN(percentage)) {
                          percentage = 0;
                      }
                      $widgetContainer.find("#pcHDLogs").prop("innerHTML", (percentage + "%"));
               }

            var _attachSubscriber = function (prop, cx) {
                var container = this;
                var propName = prop.getName();
                console.log("Property name: " + prop.getName());

                if (propName === "dataPointsImportState") {
                    if (!container.getDataPointsImportState()) {
                        //stop datapoints timer
                        dataPointsStatus(container.getSelectedDataPointsCount(),container.getCompletedDataPointsCount());
                        _startStopImporting(true);
                    }
                } else if (propName === "completedDataPointsCount") {
                    //update datapoints count text and progress percentage
                    dataPointsStatus(container.getSelectedDataPointsCount(),container.getCompletedDataPointsCount());
                } else if (propName === "alarmListImportState") {
                    if (!container.getAlarmListImportState()) {
                        //stop alarms timer
                        alarmListStatus(container.getSelectedAlarmListCount(),container.getCompletedAlarmListCount());
                        _startStopImporting(true);
                    }
                } else if(propName === "completedAlarmListCount") {
                    //update alarms count text and progress percentage
                    alarmListStatus(container.getSelectedAlarmListCount(),container.getCompletedAlarmListCount());
                } else if (propName === "hdLogsImportState") {
                    if (!container.getHdLogsImportState()) {
                        //stop hdlogs timer
                        hdLogsStatus(container.getSelectedHDLogsCount(),container.getCompletedHDLogsCount());
                        _startStopImporting(true);
                    }
                } else if (propName === "completedHDLogsCount") {
                    //update hdlogs count text and progress percentage
                    hdLogsStatus(container.getSelectedHDLogsCount(),container.getCompletedHDLogsCount());
                }
            };

            var _resetData = function () {
                selectedModules = {};
                importStatus = {};
                importList = {datapoints: 0, alarms: 0, hdlogs: 0};
            };

            var _cancelImportOperation = function(sbc_service){
                $widgetContainer.find("#importView #sbcOfflineEngineering-status-message").html(lex.get("sbc.offlineEngineering.cancel.status"));
                _updateBttnsOnCancel();
                sbc_service.invoke({
                    slot: "clearScannedInfo",
                    value: 2 //Cancel Import
                });
            };

            return {

                /**
                 *
                 * @param element: dom element the view will be added to
                 * @param params: javascript object with widget parameters required to load the view within the widget
                 */

                loadView: function (element, params) {
                    _resetData();
                    $element = $(element);
                    $widgetContainer = (params.widgetContainer) ? $(params.widgetContainer) : (element).closest(".sbcOfflineEngineering-wizard");
                    selectedModules = params.selectedModules;
                    importStatus = params.selectedModules;

                    sbcService = params.component;
                    var subscriber = params.subscriber;
                    isImportProcessCancelled = false;
                    sbcService.setIsImportCancelled(false); //reset the flag
                    $(element).append(template({
                        defaultActionMsg: lex.get("sbc.offlineEngineering.import.defaultMsg"),
                        datapoints: lex.get("sbc.offlineEngineering.select.datapoints"),
                        hdlogs: lex.get("sbc.offlineEngineering.select.hdlogs"),
                        alarms: lex.get("sbc.offlineEngineering.select.alarms"),
                        selectedAlarms: true,
                        selectedHdlogs: true
                    }));
                    $widgetContainer.find("#pcDataPoints").prop("innerHTML", "0%");
                    $widgetContainer.find("#pcAlarms").prop("innerHTML", "0%");
                    $widgetContainer.find("#pcHDLogs").prop("innerHTML", "0%");
                    $widgetContainer.find("#sbcOfflineEngineering-nextBttn").css("display","none");
                    subscriber.attach('changed', _attachSubscriber);

                    progress.prototype.buildProgress("#import-datapoints-circle", "#import-alarms-circle",
                        "#import-hdlogs-circle", "#sbcOfflineEngineering-importView");


                    var datapointsContainer = sbcService.getDataPoints();
                    var alarmsContainer = sbcService.getAlarms();
                    var hdlogsContainer = sbcService.getHdLogs();

                    //TODO - add alarms and HDLogs
                    var importInProgress = datapointsContainer.getDataPointsImportState()
                        || alarmsContainer.getAlarmListImportState()
                        || hdlogsContainer.getHdLogsImportState();


                    if (!importInProgress) {
                        var initialCount = 0;
                        subscriber.subscribe(datapointsContainer);
                        subscriber.subscribe(alarmsContainer);
                        subscriber.subscribe(hdlogsContainer);
                        progress.prototype.setDatapointsCount(initialCount + " of " + initialCount);
                        progress.prototype.setAlarmsCount(initialCount + " of " + initialCount);
                        progress.prototype.setHDLogsCount(initialCount + " of " + initialCount);
                        _startStopImporting();
                    }
                },

                /**
                 * validate user input
                 *
                 * @returns {boolean}: return true/false based on validity of user input
                 */

                validate: function () {
                    return new Promise(function (resolve) {
                        resolve(true);
                    });
                },

                /**
                 * clean up and save before going to next view
                 *
                 * @returns {Promise}: return promise to be resolved before going to next view
                 */
                next: function (event, params) {
                    return new Promise(function (resolve) {
                        params.subscriber.detach("changed", _attachSubscriber);
                        resolve();
                    });
                },

                /**
                 * clean up and save before going to previous view
                 *
                 */
                back: function (event, params) {
                    return new Promise(function (resolve) {
                        params.subscriber.detach("changed", _attachSubscriber);
                        resolve();
                    });
                },

                /**
                 * cancel operation
                 *
                 */

                cancel: function (sbc_service) {
                    sbc_service.setIsImportCancelled(false); //reset the flag
                    cancelDialog = Dialogs.showYesNo({
                        title : lex.get("sbc.offlineEngineering.dialog.confirm"),
                        content:lex.get("sbc.offlineEngineering.import.cancel.msg")
                    })
                        .yes(function (dialog) {
                            isImportProcessCancelled = true;
                            dialog.close();
                            cancelDialog = null;
                            _cancelImportOperation(sbc_service);
                            _startStopImporting(true);
                        }).no(function(dialog) {
                            isImportProcessCancelled = false;
                            dialog.close();
                            cancelDialog = null;
                    });
                },

                /**
                 * data to be passed to widget to communicate between views
                 *
                 * @returns {json}: return key value pairs of data
                 */

                getData: function () {
                    //return {scannedModules: importList};
                    return {};
                },


                params: {
                    vname: 'importView',
                    id: 'importView',
                    message: lex.get('sbc.offlineEngineering.import'),
                    backBttnMsg: lex.get('sbc.offlineEngineering.back'),
                    nextBttnMsg: lex.get('sbc.offlineEngineering.next'),
                    isCancelAvailable: true,
                    cancelBttnMsg: lex.get('sbc.offlineEngineering.cancel')
                }
            };
        };

        return importView;
    });
